@@ -1,4 +1,4 @@
-const util = require('util')
+const util = require('node:util')
 const Edge = require('../lib/edge.js')
 const OverrideSet = require('../lib/override-set.js')
 const t = require('tap')
@@ -57,6 +57,9 @@ const top = {
   addEdgeIn (edge) {
     this.edgesIn.add(edge)
   },
+  deleteEdgeIn (edge) {
+    this.edgesIn.delete(edge)
+  },
 }
 
 const a = {
@@ -71,6 +74,7 @@ const a = {
   },
   isTop: false,
   parent: top,
+  root: top,
   resolve (n) {
     return n === 'aa' ? aa : this.parent.resolve(n)
   },
@@ -80,6 +84,9 @@ const a = {
   addEdgeIn (edge) {
     this.edgesIn.add(edge)
   },
+  deleteEdgeIn (edge) {
+    this.edgesIn.delete(edge)
+  },
 }
 
 const b = {
@@ -103,6 +110,9 @@ const b = {
   addEdgeIn (edge) {
     this.edgesIn.add(edge)
   },
+  deleteEdgeIn (edge) {
+    this.edgesIn.delete(edge)
+  },
 }
 
 const bb = {
@@ -126,6 +136,9 @@ const bb = {
   addEdgeIn (edge) {
     this.edgesIn.add(edge)
   },
+  deleteEdgeIn (edge) {
+    this.edgesIn.delete(edge)
+  },
 }
 
 const aa = {
@@ -149,6 +162,9 @@ const aa = {
   addEdgeIn (edge) {
     this.edgesIn.add(edge)
   },
+  deleteEdgeIn (edge) {
+    this.edgesIn.delete(edge)
+  },
 }
 
 const c = {
@@ -172,6 +188,9 @@ const c = {
   addEdgeIn (edge) {
     this.edgesIn.add(edge)
   },
+  deleteEdgeIn (edge) {
+    this.edgesIn.delete(edge)
+  },
 }
 
 t.matchSnapshot(new Edge({
@@ -227,6 +246,16 @@ t.ok(new Edge({
 }).satisfiedBy(c), 'c@2 satisfies spec:1.x, accept:2.x')
 reset(a)
 
+t.equal(
+  (new Edge({
+    from: a,
+    type: 'prod',
+    name: 'c',
+    spec: '1.x',
+    accept: '2.x',
+  })).accept, '2.x', '.accept getter works')
+reset(a)
+
 t.ok(new Edge({
   from: a,
   type: 'prod',
@@ -353,6 +382,9 @@ const referenceTop = {
   addEdgeIn (edge) {
     this.edgesIn.add(edge)
   },
+  deleteEdgeIn (edge) {
+    this.edgesIn.delete(edge)
+  },
   overrides: new OverrideSet({
     overrides: {
       referenceGrandchild: '$referenceChild',
@@ -392,6 +424,9 @@ const referenceChild = {
     this.overrides = edge.overrides
     this.edgesIn.add(edge)
   },
+  deleteEdgeIn (edge) {
+    this.edgesIn.delete(edge)
+  },
 }
 
 new Edge({
@@ -431,6 +466,9 @@ const referenceGrandchild = {
     this.overrides = edge.overrides
     this.edgesIn.add(edge)
   },
+  deleteEdgeIn (edge) {
+    this.edgesIn.delete(edge)
+  },
 }
 
 const referenceGrandchildEdge = new Edge({
@@ -479,6 +517,9 @@ const badOverride = {
   addEdgeIn (edge) {
     this.edgesIn.add(edge)
   },
+  deleteEdgeIn (edge) {
+    this.edgesIn.delete(edge)
+  },
   overrides: new OverrideSet({
     overrides: {
       b: '1.x',
@@ -764,6 +805,9 @@ const bundleChild = {
   addEdgeIn (edge) {
     this.edgesIn.add(edge)
   },
+  deleteEdgeIn (edge) {
+    this.edgesIn.delete(edge)
+  },
 }
 
 const bundleParent = {
@@ -786,6 +830,9 @@ const bundleParent = {
   addEdgeIn (edge) {
     this.edgesIn.add(edge)
   },
+  deleteEdgeIn (edge) {
+    this.edgesIn.delete(edge)
+  },
 }
 
 const bundledEdge = new Edge({
@@ -809,6 +856,159 @@ t.same(bundledEdge.explain(), {
   from: bundleParent.explain(),
 }, 'bundled edge.explain as expected')
 
+t.test('override references find the correct root', (t) => {
+  const overrides = new OverrideSet({
+    overrides: {
+      foo: '$foo',
+    },
+  })
+
+  const root = {
+    name: 'root',
+    packageName: 'root',
+    edgesOut: new Map(),
+    edgesIn: new Set(),
+    explain: () => 'root node explanation',
+    package: {
+      name: 'root',
+      version: '1.2.3',
+      dependencies: {
+        foo: '^1.0.0',
+      },
+      overrides: {
+        foo: '$foo',
+      },
+    },
+    get version () {
+      return this.package.version
+    },
+    isTop: true,
+    parent: null,
+    overrides,
+    resolve (n) {
+      return n === 'foo' ? foo : null
+    },
+    addEdgeOut (edge) {
+      this.edgesOut.set(edge.name, edge)
+    },
+    addEdgeIn (edge) {
+      this.edgesIn.add(edge)
+    },
+    deleteEdgeIn (edge) {
+      this.edgesIn.delete(edge)
+    },
+  }
+
+  const foo = {
+    name: 'foo',
+    packageName: 'foo',
+    edgesOut: new Map(),
+    edgesIn: new Set(),
+    explain: () => 'foo node explanation',
+    package: {
+      name: 'foo',
+      version: '1.2.3',
+      dependencies: {},
+    },
+    get version () {
+      return this.package.version
+    },
+    parent: root,
+    root: root,
+    resolve (n) {
+      return n === 'bar' ? bar : this.parent.resolve(n)
+    },
+    addEdgeOut (edge) {
+      this.edgesOut.set(edge.name, edge)
+    },
+    addEdgeIn (edge) {
+      this.edgesIn.add(edge)
+    },
+    deleteEdgeIn (edge) {
+      this.edgesIn.delete(edge)
+    },
+  }
+  foo.overrides = overrides.getNodeRule(foo)
+
+  const bar = {
+    name: 'bar',
+    packageName: 'bar',
+    edgesOut: new Map(),
+    edgesIn: new Set(),
+    explain: () => 'bar node explanation',
+    package: {
+      name: 'bar',
+      version: '1.2.3',
+      dependencies: {
+        foo: '^2.0.0',
+      },
+    },
+    get version () {
+      return this.package.version
+    },
+    parent: foo,
+    root: root,
+    resolve (n) {
+      return this.parent.resolve(n)
+    },
+    addEdgeOut (edge) {
+      this.edgesOut.set(edge.name, edge)
+    },
+    addEdgeIn (edge) {
+      this.edgesIn.add(edge)
+    },
+    deleteEdgeIn (edge) {
+      this.edgesIn.delete(edge)
+    },
+  }
+  bar.overrides = foo.overrides.getNodeRule(bar)
+
+  const virtualBar = {
+    name: 'bar',
+    packageName: 'bar',
+    edgesOut: new Map(),
+    edgesIn: new Set(),
+    explain: () => 'bar node explanation',
+    package: {
+      name: 'bar',
+      version: '1.2.3',
+      dependencies: {
+        foo: '^2.0.0',
+      },
+    },
+    parent: null,
+    root: null,
+    sourceReference: bar,
+    get version () {
+      return this.package.version
+    },
+    resolve (n) {
+      return bar.resolve(n)
+    },
+    addEdgeOut (edge) {
+      this.edgesOut.set(edge.name, edge)
+    },
+    addEdgeIn (edge) {
+      this.edgesIn.add(edge)
+    },
+    deleteEdgeIn (edge) {
+      this.edgesIn.delete(edge)
+    },
+  }
+  virtualBar.overrides = overrides
+
+  const edge = new Edge({
+    from: virtualBar,
+    type: 'prod',
+    spec: '^2.0.0',
+    name: 'foo',
+    overrides: overrides.getEdgeRule({ name: 'foo', spec: '^2.0.0' }),
+  })
+
+  t.ok(edge.valid, 'edge is valid')
+  t.end()
+})
+
 t.test('shrinkwrapped and bundled deps are not overridden and remain valid', (t) => {
   const overrides = new OverrideSet({
     overrides: {
@@ -847,6 +1047,9 @@ t.test('shrinkwrapped and bundled deps are not overridden and remain valid', (t)
     addEdgeIn (edge) {
       this.edgesIn.add(edge)
     },
+    deleteEdgeIn (edge) {
+      this.edgesIn.delete(edge)
+    },
   }
 
   const foo = {
@@ -867,6 +1070,7 @@ t.test('shrinkwrapped and bundled deps are not overridden and remain valid', (t)
       return this.package.version
     },
     parent: root,
+    root,
     resolve (n) {
       return n === 'bar' ? bar : this.parent.resolve(n)
     },
@@ -876,6 +1080,9 @@ t.test('shrinkwrapped and bundled deps are not overridden and remain valid', (t)
     addEdgeIn (edge) {
       this.edgesIn.add(edge)
     },
+    deleteEdgeIn (edge) {
+      this.edgesIn.delete(edge)
+    },
   }
   foo.overrides = overrides.getNodeRule(foo)
 
@@ -895,6 +1102,7 @@ t.test('shrinkwrapped and bundled deps are not overridden and remain valid', (t)
       return this.package.version
     },
     parent: foo,
+    root,
     resolve (n) {
       return this.parent.resolve(n)
     },
@@ -904,6 +1112,9 @@ t.test('shrinkwrapped and bundled deps are not overridden and remain valid', (t)
     addEdgeIn (edge) {
       this.edgesIn.add(edge)
     },
+    deleteEdgeIn (edge) {
+      this.edgesIn.delete(edge)
+    },
   }
   bar.overrides = foo.overrides.getNodeRule(bar)
 
@@ -918,3 +1129,118 @@ t.test('shrinkwrapped and bundled deps are not overridden and remain valid', (t)
   t.ok(edge.valid, 'edge is valid')
   t.end()
 })
+
+t.test('overrideset comparison logic', (t) => {
+  const overrides1 = new OverrideSet({
+    overrides: {
+      bar: '^2.0.0',
+    },
+  })
+
+  const overrides2 = new OverrideSet({
+    overrides: {
+      bar: '^2.0.0',
+    },
+  })
+
+  const overrides3 = new OverrideSet({
+    overrides: {
+      foo: '^2.0.0',
+    },
+  })
+
+  const overrides4 = new OverrideSet({
+    overrides: {
+      foo: '^1.0.0',
+    },
+  })
+
+  const overrides5 = new OverrideSet({
+    overrides: {
+      bar: '^2.0.0',
+      foo: '^2.0.0',
+    },
+  })
+
+  const overrides6 = new OverrideSet({
+    overrides: {
+    },
+  })
+
+  const overrides7 = new OverrideSet({
+    overrides: {
+      bar: {
+        '.': '^2.0.0',
+        baz: '1.2.3',
+      },
+    },
+  })
+
+  t.ok(overrides1.isEqual(overrides1), 'overridesets are equal')
+  t.ok(overrides1.isEqual(overrides2), 'overridesets are equal')
+  t.ok(!overrides1.isEqual(overrides3), 'overridesets are different')
+  t.ok(!overrides1.isEqual(overrides5), 'overridesets are different')
+  t.ok(!overrides1.isEqual(overrides6), 'overridesets are different')
+  t.ok(!overrides1.isEqual(overrides7), 'overridesets are different')
+  t.ok(!overrides3.isEqual(overrides1), 'overridesets are different')
+  t.ok(!overrides3.isEqual(overrides4), 'overridesets are different')
+  t.ok(!overrides3.isEqual(overrides5), 'overridesets are different')
+  t.ok(!overrides4.isEqual(overrides5), 'overridesets are different')
+  t.ok(!overrides5.isEqual(overrides1), 'overridesets are different')
+  t.ok(!overrides5.isEqual(overrides3), 'overridesets are different')
+  t.ok(!overrides5.isEqual(overrides6), 'overridesets are different')
+  t.ok(!overrides6.isEqual(overrides1), 'overridesets are different')
+  t.ok(!overrides6.isEqual(overrides3), 'overridesets are different')
+  t.ok(overrides6.isEqual(overrides6), 'overridesets are equal')
+  t.ok(!overrides7.isEqual(overrides1), 'overridesets are different')
+  t.end()
+})
+
+t.test('override fallback to local when root missing dependency with from.overrides set', t => {
+  const localFrom = {
+    package: {
+      devDependencies: {
+        foo: '^1.2.3',
+      },
+    },
+    root: {
+      package: {
+        // no 'foo' defined here
+      },
+    },
+    edgesOut: new Map(),
+    edgesIn: new Set(),
+    // dummy overrides object that returns an override with isEqual defined
+    overrides: {
+      getEdgeRule (edge) {
+        return {
+          value: edge.overrides.value,
+          name: edge.overrides.name,
+          isEqual (other) {
+            return other && this.value === other.value && this.name === other.name
+          },
+        }
+      },
+    },
+    addEdgeOut (edge) {
+      this.edgesOut.set(edge.name, edge)
+    },
+    resolve () {
+      return null
+    },
+  }
+
+  const edge = new Edge({
+    from: localFrom,
+    type: 'prod',
+    name: 'foo',
+    spec: '1.x',
+    overrides: {
+      value: '$foo',
+      name: 'foo',
+    },
+  })
+
+  t.equal(edge.spec, '^1.2.3', 'should fallback to local package version from devDependencies')
+  t.end()
+})
